.module GUI

.var ubyte, DirOffset   ; In-column.
.var ubyte, FileOffset  ; In-column.
.var ubyte, SelectedCol ; Which col selected?

.var ubyte, DirScrollOffset
.var ubyte, FileScrollOffset

.var ubyte, AvailableDirs
.var ubyte, AvailableFiles

.var uword, DirNames
.var uword, FileNames

.var uword, DownloadedFileNames
.var uword, DownloadedDirNames

; ---------------------------------------------------------
; GetSelectionOffset -> Get selection rectangle offset.
; ---------------------------------------------------------
; Inputs:    a = list position.
; Outputs:   hl = offset (add gbuf).
;            c = 7
; Destroyed: af, b, de.
; ---------------------------------------------------------
GetSelectionOffset
	add a,a
	ld b,a
	add a,a
	add a,b
	ld l,a
	ld h,0
	add hl,hl
	add hl,hl
	push hl
	add hl,hl
	pop de
	add hl,de
	ld c,7
	ret
	

; ---------------------------------------------------------
; Reset -> Set up for defaults.
; ---------------------------------------------------------
; Destroyed: af, hl.
; ---------------------------------------------------------
Reset
	xor a
	ld (DirOffset),a
	ld (FileOffset),a
	ld (SelectedCol),a
	ld (AvailableDirs),a
	ld (AvailableFiles),a
	ld (DirScrollOffset),a
	ld (FileScrollOffset),a


	ld hl,saveSScreen
	ld (FileNames),hl
	ld (DownloadedFileNames),hl
	
	ld hl,saveSScreen+767
	ld (DirNames),hl
	ld (DownloadedDirNames),hl
	ret


; ---------------------------------------------------------
; Draw -> Draw the GUI interface.
; ---------------------------------------------------------
; Destroyed: af, bc, de, hl.
; ---------------------------------------------------------
Draw

	ld hl,plotSScreen
	ld de,plotSScreen+1
	ld (hl),$FF
	ld bc,12*7
	ldir
	ld (hl),$00
	ld bc,12*49
	ldir
	ld (hl),$FF
	ld bc,12*8-1
	ldir
	
	ld de,12

	; Left stripe.
	ld hl,plotSScreen+12*7
	ld c,%10000000
	ld b,55
-	ld a,(hl)
	or c
	ld (hl),a
	add hl,de
	djnz {-}
	
	; Right stripe/shadow/scrollbar.
	ld hl,plotSScreen+12*7+11
	ld a,%01 1010 11
	ld b,49
-	ld (hl),a
	xor %00 1111 00
	add hl,de
	djnz {-}
	add hl,de
	
	; Central scrollbar.
	ld hl,plotSScreen+12*12+4
	ld a,%00 1 0101 1
	ld b,39
-	ld (hl),a
	xor %00 0 1111 0
	add hl,de
	djnz {-}
	
	; Scrollbar buttons.
	ld hl,Parent.Resources.Images.ScrollButtons
	
	ld de,plotSScreen+12*7+4
	ld b,5
	call ColDir
	
	ld de,plotSScreen+12*51+4
	ld b,5
	call ColDir
	
	ld de,plotSScreen+12*7+11
	ld b,5
	call ColDir
	
	ld de,plotSScreen+12*51+11
	ld b,5
	call ColDir
	
	; Shadow details
	ld a,%01111111
	ld (plotSScreen),a
	ld (plotSScreen+12*62),a
	rlca
	ld (plotSScreen+11+12*1),a
	ld (plotSScreen+767),a
	add a,a
	ld (plotSScreen+11),a
	ld a,%00111111
	ld (plotSScreen+12*63),a
	

	ld hl,DirOffset
	ld de,DirScrollOffset
	ld a,(AvailableDirs)
	ld b,a
	call FixScroll

	ld hl,FileOffset
	ld de,FileScrollOffset
	ld a,(AvailableFiles)
	ld b,a
	call FixScroll

	; Draw dirs
	ld a,(AvailableDirs)
	or a
	jr z,NoDirs
	cp 8
	jr c,{+}
	ld a,8
+	ld b,a

	ld hl,1+8*256
	ld (penCol),hl	
	ld hl,(DirNames)
	ld a,(DirScrollOffset)
	push bc
	call FindNthString
	pop bc
	
-	push bc

	ld a,(hl)
	cp '.'
	jr nz,{+}
	dec hl
	ld a,(hl)
	inc hl
	cp '.'
	jr nz,{+}
	
	dec hl
	dec hl
	dec hl
	push hl
	
	ld ix,Parent.Resources.Images.DirUp
	ld a,(penRow)
	ld l,a
	ld a,(penCol)
	inc a
	ld b,5
	ld c,2
	call ionLargeSprite
	
	pop hl	
	jr DrawnFolderUp
+

	call Parent.FixedWidth.PutSReversed
DrawnFolderUp
	ld a,(penRow)
	add a,6
	ld (penRow),a
	ld a,1
	ld (penCol),a
	pop bc
	djnz {-}
	
NoDirs
	
	; Draw files/folders
	ld a,(AvailableFiles)
	or a
	jr z,NoFiles
	cp 8
	jr c,{+}
	ld a,8
+	ld b,a

	ld hl,40+8*256
	ld (penCol),hl
	
	ld hl,(FileNames)
	ld a,(FileScrollOffset)
	push bc
	call FindNthString
	pop bc

-	push bc
	call Parent.FixedWidth.PutS
	ld a,(penRow)
	add a,6
	ld (penRow),a
	ld a,40
	ld (penCol),a
	pop bc
	djnz {-}
	
NoFiles
	
	; Draw directory selection rectangle.
	
	ld a,(SelectedCol)
	or a
	jr nz,FileSelected
	ld a,(DirOffset)
	call GetSelectionOffset
	
	ld de,plotSScreen+12*7
	add hl,de
	ld de,8
	
	
--	ld a,(hl)
	xor %01111111
	ld (hl),a
	inc hl
	ld b,3
-	ld a,(hl)
	cpl
	ld (hl),a
	inc hl
	djnz {-}
	ld a,(hl)
	xor %11000000
	ld (hl),a
	add hl,de
	dec c
	jr nz,{--}
	jr DoneDirSelected
	
FileSelected
	; Draw file selection rectangle.
	ld a,(FileOffset)
	call GetSelectionOffset
	
	ld de,plotSScreen+12*7+5
	add hl,de
	ld de,6
	
--	ld b,6
-	ld a,(hl)
	cpl
	ld (hl),a
	inc hl
	djnz {-}
	ld a,(hl)
	xor %10000000
	ld (hl),a
	add hl,de
	dec c
	jr nz,{--}
DoneDirSelected

	; Fill in the scroll bars - dir
	ld a,(DirOffset)
	ld b,a
	ld a,(DirScrollOffset)
	add a,b
	ld b,a
	ld a,(AvailableDirs)

	call CalculateScrollBar
	
	ld de,4
	add hl,de
	ld de,12
	
	ld a,(hl)
	or %00 0 1111 0
	ld (hl),a	
	add hl,de
	
-	ld a,(hl)
	and ~%00 0 1111 0
	ld (hl),a
	add hl,de
	djnz {-}
	
	ld a,(hl)
	or %00 0 1111 0
	ld (hl),a

	; Fill in the scroll bars - file
	ld a,(FileOffset)
	ld b,a
	ld a,(FileScrollOffset)
	add a,b
	ld b,a
	ld a,(AvailableFiles)
	
	call CalculateScrollBar
	
	ld de,11
	add hl,de
	ld de,12
	
	ld a,(hl)
	or %00 1111 00
	ld (hl),a	
	add hl,de
	
-	ld a,(hl)
	and ~%00 1111 00
	ld (hl),a
	add hl,de
	djnz {-}
	
	ld a,(hl)
	or %00 1111 00
	ld (hl),a

	; Draw status bar.
	ld hl,57*256+1
	ld (penCol),hl
	ld hl,Parent.Resources.Strings.Ready
	ld a,(Parent.VMusic2.LastEvent)
	or a
	jr z,{+}
	ld hl,Parent.Resources.Strings.Busy
+
	call Parent.FixedWidth.PutS

	ret

; ---------------------------------------------------------
; ColDir -> "LDIR" in columns, not rows.
; ---------------------------------------------------------
; Inputs:    de -> destination.
;            hl -> source.
;            b  =  number of bytes to copy.
; ---------------------------------------------------------
ColDir
	ex de,hl
-	ld a,(de)
	inc de
	ld (hl),a
	push de
	ld de,12
	add hl,de
	pop de
	djnz {-}
	ex de,hl
	ret	
	

; ---------------------------------------------------------
; FixScroll -> Fix overflowing scrollbars.
; ---------------------------------------------------------
; Inputs:    hl -> offset (column, so 0 = top of screen).
;            de -> scrollbar offset.
;            b  =  scroll maximum.
; ---------------------------------------------------------
FixScroll
	ld a,b
	or a
	jr nz,{+}
	ld (de),a
	ld (hl),a
	ret
+
	ld a,(de)
	sub b
	neg
	cp (hl)
	jr nz,{+}
	dec a
	ld (hl),a
	ret

+

	ld a,(hl)
	cp 8
	jr nz,{+}
	dec (hl)
	
	ld a,(de)
	add a,(hl)
	cp b
	ret z
	ld a,(de)
	inc a
	ld (de),a
	ret	
+	cp -1
	ret nz
	inc (hl)
	
	ld a,(de)
	or a
	ret z
	dec a
	ld (de),a
	ret
	


; ---------------------------------------------------------
; RefreshFileList -> Get file listing from drive.
; ---------------------------------------------------------
; Destroyed: af, bc, de, hl.
; ---------------------------------------------------------
RefreshFileList

	ld hl,saveSScreen
	ld (FileNames),hl
	ld (DownloadedFileNames),hl
	ld hl,saveSScreen+767
	ld (DirNames),hl
	ld (DownloadedDirNames),hl


	xor a
	ld (DirOffset),a
	ld (FileOffset),a
	ld (AvailableDirs),a
	ld (AvailableFiles),a

	jp Parent.VMusic2.Directory.GetListing

; ---------------------------------------------------------
; GetSelectedName -> Get the selected item's name.
; ---------------------------------------------------------
; Outputs:   OP1 = name.
; ---------------------------------------------------------
GetSelectedName
	ld a,(SelectedCol)
	or a
	jr nz,UseFileName

	ld de,-1
	ld hl,(DirNames)
	ld a,(DirOffset)
	ld b,a
	ld a,(DirScrollOffset)
	jr UsedDirName
UseFileName
	ld de,+1
	ld hl,(FileNames)
	ld a,(FileOffset)
	ld b,a
	ld a,(FileScrollOffset)
UsedDirName
	add a,b
	ld b,a
	or a
	jr z,{+}
	
-	ld a,(hl)
	add hl,de
	or a
	jr nz,{-}
	djnz {-}
	
+	ld bc,OP1

-	ld a,(hl)
	ld (bc),a
	add hl,de
	inc bc
	or a
	jr nz,{-}
	ret


; ---------------------------------------------------------
; FindNthString -> Find nth string in a list.
; ---------------------------------------------------------
; Inputs:   a  =  index.
;           hl -> strings.
; Outputs:  hl -> nth string.
; ---------------------------------------------------------
FindNthString
	or a
	ret z
	ld b,a
-	ld a,(hl)
	inc hl
	or a
	jr nz,{-}
	djnz {-}
	ret

; ---------------------------------------------------------
; CalculateScrollBar -> Get scroll bar position.
; ---------------------------------------------------------
; Inputs:   a = total items in list.
;           b = index.
; Outputs:  b = height of thumb.
;           hl -> top of thumb (on buffer).
;           de = 12
; ---------------------------------------------------------
CalculateScrollBar
	; Scroll bar height = 39 / total items.
	; Cap to minimum of 4:
	or a
	jr nz,{+}
	inc a
+
	ld c,a
	
	ld hl,41
	push bc
	bcall(_DivHLByA)
	pop bc
	
	ld e,0
	ld a,4
	sub l
	jr c,{+}
	ld l,4
	ld e,a
+	
	; l = height
	push hl

	; (index / num items) * height
	; = (index * height) / num items
	
	ld hl,$0000
	ld a,b
	or a
	jr z,ScrollBIsZero
	
	ld a,42
	sub e
	ld l,a
	ld h,b
	push bc
	bcall(_HTimesL)
	pop bc
	ld a,c
	bcall(_DivHLByA)

	ld h,0
	add hl,hl
	add hl,hl
	ld d,h \ ld e,l
	add hl,hl
	add hl,de
ScrollBIsZero
	ld de,plotSScreen+12*11
	add hl,de
	
	pop bc
	ld b,c
	ld de,12
	dec b
	dec b
	ret


; ---------------------------------------------------------
; ShowDialogDitherDelay -> Show a dialog on a dithered
;                          background and wait for a key.
; ---------------------------------------------------------
; Inputs:   a = dialog index.
; ---------------------------------------------------------
ShowDialogDitherDelay
	push af
	call Parent.Dialogs.Dither
	pop af
	; Run on...

; ---------------------------------------------------------
; ShowDialogDelay -> Show a dialog and wait for a key.
; ---------------------------------------------------------
; Inputs:   a = dialog index.
; ---------------------------------------------------------
ShowDialogDelay
	call ShowDialog
	jp Parent.Dialogs.WaitKey

; ---------------------------------------------------------
; ShowDialog -> Show a dialog.
; ---------------------------------------------------------
; Inputs:   a = dialog index.
; ---------------------------------------------------------
ShowDialog
	ld hl,Parent.Resources.Dialogs
	call Parent.Dialogs.DrawMessageFromTable
	call ionFastCopy
	ret	
	
.endmodule