.module Directory

; ---------------------------------------------------------
; GetListing -> Get a directory listing.
; ---------------------------------------------------------
; Inputs:   None.
; Outputs:  z = set up successfully; nz = failed.
; Destroys: af, hl.
; Events:   Will trigger an event for each directory item
;           and a single success event.
;           Files will just be listed with their name;
;           directories will have their name followed by a
;           space and 'DIR'. Use GetIsDir to check the name
;           and GetNameOnly to strip off the ' DIR' ending.
; ---------------------------------------------------------
GetListing
	ld a,Parent.Events.Directory.Listing
	ld hl,Commands.List
	jp Parent.SetUpEvent

; ---------------------------------------------------------
; Change -> Change to a different directory.
; ---------------------------------------------------------
; Inputs:   OP1 = name of directory to change to.
; Outputs:  z = success, nz = failure.
; Destroys: af, hl.
; Events:   Single success event.
; ---------------------------------------------------------	
Change
	ld a,Parent.Events.Directory.Change
	ld hl,Commands.Change
	jp Parent.SetUpEventOP1Argument

; ---------------------------------------------------------
; GetIsDirectory -> Check if an item name is a directory.
; ---------------------------------------------------------
; Inputs:   hl -> name to check.
; Outputs:  z = is a directory; nz = isn't a directory.
; Destroys: af.
; ---------------------------------------------------------
GetIsDirectory
	push hl
	push de
	push bc
-	ld a,(hl)
	or a
	jr z,{+}
	inc hl
	jr {-}
+
	ld de,Dir
	ld b,DirLength
-	dec hl
	ld a,(de)
	cp (hl)
	jr nz,{+}
	inc de
	djnz {-}	
+	pop bc
	pop de
	pop hl
	ret
Dir .db "RID " \ DirLength = $-Dir


; ---------------------------------------------------------
; GetNameOnly -> Get the name of a directory.
; ---------------------------------------------------------
; Inputs:   hl -> name to fix.
; Outputs:  None.
; Destroys: af.
; Remarks:  Overwrites the existing string. Replace the NUL
;           terminator with a space to get back to the
;           original.
; ---------------------------------------------------------
GetNameOnly
	push hl
-	ld a,(hl)
	or a
	jr z,NoSpaceFound
	cp ' '
	jr z,{+}
	inc hl
	jr {-}
+
	ld (hl),0
NoSpaceFound
	pop hl
	ret



.module Commands
	List   .db "DIR", 0
	Delete .db "DLD", 0
	Create .db "MKD", 0
	Change .db "CD", 0
.endmodule

.endmodule