.module Music

.var ubyte, LastRegister

; ---------------------------------------------------------
; Play -> Play a music file.
; ---------------------------------------------------------
; Inputs:   OP1 = name of file to play.
; Outputs:  z = success, nz = failure.
; Destroys: af, hl.
; Events:   Single success event.
; ---------------------------------------------------------
Play
	ld a,Parent.Events.Music.Play
	ld hl,Commands.Play
	call Parent.SetUpEventOP1Argument
	ret nz
	xor a
	ld (Parent.LastEvent),a
	ret


; ---------------------------------------------------------
; WriteRegister -> Write to a VS1003 register.
; ---------------------------------------------------------
; Inputs:   a  = register number.
;           hl = new register value.
; Outputs:  z = success, nz = failure.
; Destroys: af, hl, OP1.
; Events:   Single success event.
; ---------------------------------------------------------
WriteRegister	
	ld (LastRegister),a
	ld (OP1+1),a
	ld (OP1+2),hl
	ld a,3
	ld (OP1+0),a
	ld a,Parent.Events.Music.WriteRegister
	ld hl,Commands.WriteReg
	jp Parent.SetUpEventOP1LengthPrefixedArgument


; ---------------------------------------------------------
; ReadRegister -> Read from a VS1003 register.
; ---------------------------------------------------------
; Inputs:   a  = register number.
; Outputs:  z = success, nz = failure.
; Destroys: af, hl, OP1.
; Events:   Data event containing two-byte register value.
;           Single success event.
; ---------------------------------------------------------
ReadRegister
	ld (LastRegister),a
	ld (OP1+0),a
	xor a
	ld (OP1+1),a
	ld a,Parent.Events.Music.ReadRegister
	ld hl,Commands.ReadReg
	jp Parent.SetUpEventOP1Argument

; ---------------------------------------------------------
; Stop -> Stop playing a music file.
; ---------------------------------------------------------
; Inputs:   None.
; Outputs:  z = success, nz = failure.
; Destroys: af, hl.
; Events:   Single success event.
; ---------------------------------------------------------
Stop
	ld a,Parent.Events.Music.Stop
	ld hl,Commands.Stop
	jp Parent.SetUpEvent

.module Commands
	Play     .db "VPF", 0
	PlayAll  .db "V3A", 0
	Stop     .db "VST", 0
	Next     .db "VSF", 0
	Previous .db "VSB", 0
	WriteReg .db "VWR" ,0
	ReadReg  .db "VRD", 0
.endmodule

.module Registers
	Mode       = $00 ; SCI_MODE        - Mode control.
	Status     = $01 ; SCI_STATUS      - Status of VS1003.
	Bass       = $02 ; SCI_BASS        - Built-in bass/treble enhancer.
	ClockF     = $03 ; SCI_CLOCKF      - Clock freq+multiplier.
	DecodeTime = $04 ; SCI_DECODE_TIME - Decode time in seconds.
	AuData     = $05 ; SCI_AUDATA      - Misc. audio data.
	WRam       = $06 ; SCI_WRAM        - RAM write/read.
	WRamAddr   = $07 ; SCI_WRAMADDR    - Base address for RAM write/read.
	HDat0      = $08 ; SCI_HDAT0       - Stream header data 0 (read-only).
	HDat1      = $09 ; SCI_HDAT1       - Stream header data 1 (read-only).
	AIAddr     = $0A ; SCI_AIADDR      - Start address of application.
	Vol        = $0B ; SCI_VOL         - Volume control.
	AICtrl0    = $0C ; SCI_AICTRL0     - Application control register 0.
	AICtrl1    = $0D ; SCI_AICTRL1     - Application control register 1.
	AICtrl2    = $0E ; SCI_AICTRL2     - Application control register 2.
	AICtrl3    = $0F ; SCI_AICTRL3     - Application control register 3.
	
	.module Mode
		Diff       = 1 <<  0 ; SM_DIFF       - Differential.
		SetToZero  = 1 <<  1 ; SM_SETTOZERO  - Set to zero.
		Reset      = 1 <<  2 ; SM_RESET      - Soft reset.
		OutOfWav   = 1 <<  3 ; SM_OUTOFWAV   - Jump out of WAV decoding.
		PDown      = 1 <<  4 ; SM_PDOWN      - Powerdown.
		Tests      = 1 <<  5 ; SM_TESTS      - Allow SDI tests
		Stream     = 1 <<  6 ; SM_STREAM     - Stream mode.
		SetToZero2 = 1 <<  7 ; SM_SETTOZERO2 - Set to zero.
		DAct       = 1 <<  8 ; SM_DACT       - DCLK active edge.
		SdiOrd     = 1 <<  9 ; SM_SDIORD     - SDI bit order.
		SdiShare   = 1 << 10 ; SM_SDISHARE   - Share SDI chip select.
		SdiNew     = 1 << 11 ; SM_SDINEW     - VS1002 native SPI modes.
		ADPcm      = 1 << 12 ; SM_ADPCM      - ADPCM recording active.
		ADPcmHp    = 1 << 13 ; SM_ADPCM_HP   - ADPCM high-pass filter active.
		LineIn     = 1 << 14 ; SM_LINE_IN    - ADPCM recording selector.
	.endmodule
	
.endmodule

.endmodule